local name, ns = ...
local debug = ns.debug

local Addon = _G[name]
local tq = LibStub("LibTalentQuery-1.0")

local talentSpecs = {}
local talentSpecNames = {}
local talentSpecIcons = {}
local function updatetalent(event, name, realm)
	local name = name .. (realm or "")
	local data = talentSpecs[name] or {[1] = {}, [2] = {}, [3] = {},}
	local specName = { NONE, 0 }
	local isInspect = name ~= UnitName("player")
	for i = 1, 3 do
		local active = GetActiveTalentGroup(isInspect)
		local _, name, _, icon, points = GetTalentTabInfo(i, isInspect, false, active)
		data[i][1] = name
		data[i][2] = points
		data[i][3] = icon
		if points > specName[2] then
			specName[1] = name
			specName[2] = points
			specName[3] = icon
		elseif points == specName[2] then
			specName[1] = "Hybrid"
		end
	end
	talentSpecs[name] = data
	talentSpecNames[name] = specName[1]
	talentSpecIcons[name] = specName[3]
end
tq.RegisterCallback(Addon, "TalentQuery_Ready", updatetalent)


local MIN_TIME_UNTIL_RESCAN = 600
local MIN_TIME_BETWEEN_QUERY = 5
local lastQuery = 0
local lastscan = {}

local function query(unit)
	local name = UnitName(unit)
	if not name then return end
	if not CanInspect(unit) or UnitLevel(unit) < 10 then return end
	local curTime = GetTime()
	if (curTime - lastQuery) < MIN_TIME_BETWEEN_QUERY then
		debug("next scan possible in ", (lastQuery + MIN_TIME_BETWEEN_QUERY) - curTime)
		return
	end
	if (curTime - (lastscan[name] or -MIN_TIME_UNTIL_RESCAN)) < MIN_TIME_UNTIL_RESCAN then
		debug("next scan for this player in ", (lastscan[name] + MIN_TIME_UNTIL_RESCAN) - curTime)
		return
	end
	tq:Query(unit)
	lastQuery = curTime
	lastscan[name] = curTime
end

local function TalentSpec(unit)
	if not UnitIsPlayer(unit) then
		return
	end
	local name = UnitName(unit)
	if not name then return end
	query(unit)
	local spec = talentSpecs[name]
	if spec then
		return ("%d/%d/%d"):format(spec[1][2], spec[2][2], spec[3][2])
	else
		if CanInspect(unit) and UnitLevel(unit) >= 10 then
			return "Scanning Talents..."
		end
	end
end
Addon.ScriptEnv.TalentSpec = TalentSpec

local function TalentSpecName(unit)
	if not UnitIsPlayer(unit) then
		return
	end
	local name = UnitName(unit)
	if not name then return end
	query(unit)
	return talentSpecNames[name]
end
Addon.ScriptEnv.TalentSpecName = TalentSpecName

local function TalentSpecIcon(unit)
	if not UnitIsPlayer(unit) then
		return
	end
	local name = UnitName(unit)
	if not name then return end
	query(unit)
	return talentSpecIcons[name]
end
Addon.ScriptEnv.TalentSpecIcon = TalentSpecIcon
